@@ -1,5 +1,9 @@
 #' @import lme4 methods
-#' @importFrom stats dnorm na.omit pchisq pnorm qnorm quantile resid summary.glm
+#' @importFrom stats dnorm na.omit pchisq pnorm qnorm quantile resid summary.glm binomial predict
+#' residuals residuals.glm setNames weighted.mean
+#' @importFrom grDevices dev.flush dev.hold devAskNewPage
+#' @importFrom utils capture.output stack
+
 
 # Extending glmerMod to contain RR parameters for plot and summary functions
 RRglmerMod <- setClass(
@@ -28,13 +32,15 @@ RRglmerMod <- setClass(
 #' @param link
 #' a glm link function for binary outcomes. Must be a function name.
 #' Available options: "RRlink.logit", "RRlink.probit", "RRlink.cloglog" and "RRlink.cauchit"
+#' @param item
+#' optional item identifier for long-format data.
 #' @param RRmodel
-#' the Randomized Response model. Can be a single value or a vector of models.
+#' the Randomized Response model, defined per case.
 #' Available options: "DQ", "Warner", "Forced", "UQM", "Crosswise", "Triangular" and "Kuk"
 #' @param p1
-#' the Randomized Response parameter p1. Must be 0 <= p1 <= 1.
+#' the Randomized Response parameter p1, defined per case. Must be 0 <= p1 <= 1.
 #' @param p2
-#' the Randomized Response parameter p2. Must be 0 <= p2 <= 1.
+#' the Randomized Response parameter p2, defined per case. Must be 0 <= p2 <= 1.
 #' @param data
 #' a data frame containing the variables named in \code{\link{formula}} as well as the Randomized Response model and parameters.
 #' If the required information cannot be found in the data frame, or if no data frame is given, then the variables are taken
@@ -66,39 +72,28 @@ RRglmerMod <- setClass(
 #'          etastart = rep(0.01, nrow(Plagiarism)),
 #'          control = glmerControl(optimizer = "Nelder_Mead", tolPwrss = 1e-03), nAGQ = 1)
 #' summary(out)
-RRglmer <- function (formula, link, RRmodel, p1, p2, data, control = glmerControl(), na.action = "na.omit", ...) {
+RRglmer <- function (formula, item, link, RRmodel, p1, p2, data, control = glmerControl(), na.action = "na.omit", ...) {
 
-  # Try to find the RR parameters in the data set first
-  tryCatch(RRmodel <- eval(substitute(RRmodel), data), error=function(e) NULL)
-  tryCatch(p1 <- eval(substitute(p1), data), error=function(e) NULL)
-
-  if(!missing(p2))
-    tryCatch(p2 <- eval(substitute(p2), data), error=function(e) NULL)
-  else
-    p2 <- rep(1, length(p1))
-
-  if(!(length(RRmodel) == length(p1) && length(RRmodel) == length(p2)))
-    stop("RR parameter vectors are not of same length.")
-
-  if (!(length(RRmodel) == 1 || length(RRmodel) == nrow(data)))
-    stop("Length of RR parameters does not match 1 or number of elements in the data.")
-
-  if (any(is.na(RRmodel) | is.na(p1) | is.na(p2)))
-    stop("RR parameters must be specified for each case.")
+  # Create model frame
+  varnames <- all.vars(formula)
+  mf <- match.call(expand.dots = FALSE)
+  mf$formula <- paste(varnames[1], "~", paste(varnames[2:length(varnames)], collapse = "+"), sep = "") # model frame can't handle mixed-effect formulas
+  print(mf)
+  m <- match(c("formula", "data", "item", "RRmodel", "p1", "p2", "na.action"), names(mf), 0)
+  mf <- mf[c(1, m)]
+  mf$drop.unused.levels <- TRUE
+  mf[[1]] <- quote(stats::model.frame)
+  mf <- eval(mf, parent.frame())
+  if(is.null(mf$'(item)'))
+    mf$'(item)' <- rep("Single item", nrow(mf))
+  if(is.null(mf$'(p2)'))
+    mf$'(p2)' <- rep(NA, nrow(mf))
 
   # Translate p1, p2 to c, d for the chosen RR model
-  RRparameters <- getRRparameters(RRmodel, p1, p2)
+  RRparameters <- getRRparameters(mf$'(RRmodel)', mf$'(p1)', mf$'(p2)')
 
   # Create a dataset containing the variables of the formula and the the RR parameters
-  varnames <- all.vars(formula)
-  RRdata <- data.frame(data[,varnames], "RRmodel" = RRmodel, "c" = RRparameters$c, "d" = RRparameters$d, "p1" = p1, "p2" = p2)
-
-  # Debug
-  #View(RRdata)
-
-  # If NA's shall be deleted by glm(), we must do the same for the RR paramters
-  if (na.action == "na.omit" || na.action == "na.exclude")
-    RRdata <- na.omit(RRdata)
+  RRdata <- data.frame(mf[,varnames], "Item" = mf$'(item)', "RRmodel" = mf$'(RRmodel)', "c" = RRparameters$c, "d" = RRparameters$d, "p1" = mf$'(p1)', "p2" = mf$'(p2)')
 
   # Must be a data frame
   df <- as.data.frame(data)
@@ -107,7 +102,7 @@ RRglmer <- function (formula, link, RRmodel, p1, p2, data, control = glmerContro
   glmlink <- match.fun(link)
 
   # Create call object with given arguments
-  cl <- call("glmer", formula = formula, family = quote(binomial(link = glmlink(RRdata$c, RRdata$d))), data = quote(df),
+  cl <- call("glmer", formula = formula, family = quote(RRbinomial(link = glmlink(RRdata$c, RRdata$d), c = RRdata$c, d = RRdata$d)), data = quote(df),
              control = control, na.action = na.action)
 
   # Make sure that additional arguments are passed
@@ -121,8 +116,8 @@ RRglmer <- function (formula, link, RRmodel, p1, p2, data, control = glmerContro
   # Returns lme4::glmerMod object
   output <- eval(cl)
 
-  # Put RR parameters in a list
-  RRparam <- list(RRmodel = RRdata$RRmodel, p1 = RRdata$p1, p2 = RRdata$p2, c = RRdata$c, d = RRdata$d)
+  # Put item identifier and RR parameters in a list
+  RRparam <- list(Item = RRdata$Item, RRmodel = RRdata$RRmodel, p1 = RRdata$p1, p2 = RRdata$p2, c = RRdata$c, d = RRdata$d)
 
   # Add a slot for the RR parameters
   outputRR <- as(output, "RRglmerMod")
